/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright 2016, Olivier MATZ <zer0@droids-corp.org>
 */

/**
 * @addtogroup ecoli_nodes
 * @{
 */

#ifndef ECOLI_INIT_
#define ECOLI_INIT_

#include <sys/queue.h>

#include <ecoli_log.h>
#include <ecoli_node.h>

/**
 * Register initialization and exit callbacks. These callbacks are
 * ordered by priority: for initialization, the lowest priority is called
 * first. For exit, the callbacks are invoked in reverse order.
 */
#define EC_INIT_REGISTER(t)						\
	static void ec_init_init_##t(void);				\
	static void __attribute__((constructor, used))			\
	ec_init_init_##t(void)						\
	{								\
		 ec_init_register(&t);					\
	}

/**
 * Type of init function. Return 0 on success, -1 on error.
 */
typedef int (ec_init_t)(void);

/**
 * Type of exit function.
 */
typedef void (ec_exit_t)(void);

TAILQ_HEAD(ec_init_list, ec_init);

/**
 * A structure describing a test case.
 */
struct ec_init {
	TAILQ_ENTRY(ec_init) next;  /**< Next in list. */
	ec_init_t *init;            /**< Init function. */
	ec_exit_t *exit;            /**< Exit function. */
	unsigned int priority;      /**< Priority (0=first, 99=last) */
};

/**
 * Register an initialization function.
 *
 * @param test
 *   A pointer to a ec_init structure to be registered.
 */
void ec_init_register(struct ec_init *test);

/**
 * Initialize ecoli library.
 *
 * Must be called before any other function from libecoli, except
 * ec_malloc_register().
 *
 * @return
 *   0 on success, -1 on error (errno is set).
 */
int ec_init(void);

/**
 * Uninitialize ecoli library.
 */
void ec_exit(void);

#endif

/** @} */
